﻿
' TODO - Main List
' --------------------------------------------------------------------

' Done version 1.6
' --------------------------------------------------------------------
' Slots to control Amplitude, Shift, Frequency and Waveform
' Frequency changes without glitches
' ECG and OCG test signals
' Five user defined waveforms from text files

' Done version 1.5
' --------------------------------------------------------------------
'  Hi precision frequency generation with the new class AccurateTimer  

Public Class Form1

    Private Sub Form1_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        EventsAreEnabled = False
        Text = AppTitleAndVersion()
        '
        Combo_InitFromEnum(cmb_Shape, GetType(WaveShapes))
        cmb_Shape.SelectedIndex = 0
        '
        Load_INI()
        Load_ECG_OCG_Data()
        SetWaveShape()
        SetSlotParams()
        SetParams()
        '
        AccTimer.Interval = 2
        AccTimer.Start()
        '
        Timer1.Interval = 10
        Timer1.Start()
        Timer2.Interval = 100
        Timer2.Start()
        '
        ' ----------------------------------------------------------- 
        ' Do not use PriorityClass "Above normal" or "Real time"
        '  because this could degrade the AccurateTimer
        ' -----------------------------------------------------------
        ' Dim Proc As Process = Process.GetCurrentProcess()
        ' Proc.PriorityClass = ProcessPriorityClass.Normal
        ' -----------------------------------------------------------
        EventsAreEnabled = True
        Refresh()
        Me.Opacity = 1
    End Sub

    Private Sub Form1_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing
        AccTimer.Stop()
        EventsAreEnabled = False
        Save_INI()
        If AccTimer IsNot Nothing Then AccTimer.Stop()
        Slots.WriteSlot(SlotOut, NAN_Sleep)
    End Sub

    Private Sub Form_LocationChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.LocationChanged
        If Not EventsAreEnabled Then Exit Sub
        LimitFormPosition(Me)
    End Sub


    ' ==============================================================
    '   SET SLOT PARAMS - Using TextChanged and CheckedChanged
    ' ==============================================================
    Private Sub txt_TextSlotChanged(ByVal sender As System.Object, _
                               ByVal e As System.EventArgs) _
                               Handles txt_SlotOutput.TextChanged, _
                                       txt_SlotAmplitude.TextChanged, _
                                       txt_SlotShift.TextChanged, _
                                       txt_SlotFrequency.TextChanged, _
                                       txt_SlotWaveform.TextChanged
        If Not EventsAreEnabled Then Exit Sub
        ChangedParams = True
        SetSlotParams()
    End Sub
    Private Sub SetSlotParams()
        ' ----------------------------------------------- slots
        SlotOut = txt_SlotOutput.NumericValueInteger
        SlotAmp = txt_SlotAmplitude.NumericValueInteger
        SlotShift = txt_SlotShift.NumericValueInteger
        SlotFreq = txt_SlotFrequency.NumericValueInteger
        SlotWaveform = txt_SlotWaveform.NumericValueInteger
        If SlotAmp >= 0 Then
            lbl_Amplitude.Enabled = False
            txt_Amplitude.Enabled = False
        Else
            lbl_Amplitude.Enabled = True
            txt_Amplitude.Enabled = True
        End If
        If SlotShift >= 0 Then
            lbl_Shift.Enabled = False
            txt_Shift.Enabled = False
        Else
            lbl_Shift.Enabled = True
            txt_Shift.Enabled = True
        End If
        If SlotFreq >= 0 Then
            lbl_Frequency.Enabled = False
            txt_Frequency.Enabled = False
        Else
            lbl_Frequency.Enabled = True
            txt_Frequency.Enabled = True
        End If
        If SlotWaveform >= 0 Then
            cmb_Shape.Enabled = False
            cmb_Shape.BackColor = Color.FromArgb(220, 220, 220)
        Else
            cmb_Shape.Enabled = True
            cmb_Shape.BackColor = Color.MintCream
        End If
    End Sub


    ' ==============================================================
    '   SET PARAMS - Using TextChanged and CheckedChanged
    ' ==============================================================
    Private ChangedParams As Boolean
    Private Sub txt_TextChanged(ByVal sender As System.Object, _
                                ByVal e As System.EventArgs) _
                                Handles txt_Amplitude.TextChanged, _
                                        txt_Shift.TextChanged, _
                                        txt_Frequency.TextChanged, _
                                        chk_ConstantSpeed.CheckedChanged, _
                                        txt_MinFreq.TextChanged
        If Not EventsAreEnabled Then Exit Sub
        ChangedParams = True
        SetParams()
    End Sub

    Private Sub SetParams()
        ' ----------------------------------------------- params
        Amplitude = CSng(txt_Amplitude.NumericValue)
        Shift = CSng(txt_Shift.NumericValue)
        NewFrequency = CSng(txt_Frequency.NumericValue)
        ConstantSpeed = chk_ConstantSpeed.Checked
        MinFreq = CSng(txt_MinFreq.NumericValue)
        ' ----------------------------------------------- constant speed
        If ConstantSpeed Then
            If MinFreq > NewFrequency Then
                NewFrequency = MinFreq
                txt_Frequency.NumericValue = MinFreq
                txt_Frequency.Refresh()
            End If
            txt_Frequency.MinValue = MinFreq
            lbl_MinFreq.Enabled = True
            txt_MinFreq.Visible = True
        Else
            lbl_MinFreq.Enabled = False
            txt_MinFreq.Visible = False
            txt_Frequency.MinValue = 0.01
        End If
        ' ------------------------------------------------
        UpdateTrackbar()
    End Sub

    ' ==============================================================
    '   SAVE INI - Using MouseLeave
    ' ==============================================================
    Private Sub Params_MouseLeave(ByVal sender As Object, _
                                  ByVal e As System.EventArgs) _
                                        Handles txt_SlotOutput.MouseLeave, _
                                                txt_SlotAmplitude.MouseLeave, _
                                                txt_SlotShift.MouseLeave, _
                                                txt_SlotFrequency.MouseLeave, _
                                                txt_SlotWaveform.MouseLeave, _
                                                txt_Amplitude.MouseLeave, _
                                                txt_Shift.MouseLeave, _
                                                chk_ConstantSpeed.MouseLeave, _
                                                txt_MinFreq.MouseLeave, _
                                                txt_Frequency.MouseLeave
        If Not EventsAreEnabled Then Return
        If ChangedParams Then
            ChangedParams = False
            Save_INI()
        End If
    End Sub

    ' ==============================================================
    '  Combo Methods - With SaveINI
    ' ==============================================================
    Private Sub Combo_DropDown(ByVal sender As Object, ByVal e As System.EventArgs) Handles cmb_Shape.DropDown
        CType(sender, ComboBox).ItemHeight = 16
    End Sub
    Private Sub cmb_Shape_DropDownClosed(ByVal sender As Object, ByVal e As System.EventArgs) Handles cmb_Shape.DropDownClosed
        CType(sender, ComboBox).ItemHeight = 12
    End Sub
    Private Sub Combo_SelectedIndexChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles cmb_Shape.SelectedIndexChanged
        If Not EventsAreEnabled Then Return
        SetWaveShape()
        Save_INI()
    End Sub
    Private Sub SetWaveShape()
        WaveShape = CType(cmb_Shape.SelectedIndex, WaveShapes)
    End Sub

    ' ==============================================================================================================
    '   TrackBars
    ' ==============================================================================================================
    Private Sub MyTrackBar1_Scroll(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyTrackBar1.Scroll
        If Not EventsAreEnabled Then Return
        WriteValueToSlot(MyTrackBar1.Value)
    End Sub

    Private Sub UpdateTrackbar()
        If WaveShape <> WaveShapes.Manual Then
            EventsAreEnabled = False
            MyTrackBar1.Value = CInt(NewValue)
            MyTrackBar1.Refresh()
            EventsAreEnabled = True
        End If
    End Sub

    Private Sub Timer1_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Timer1.Tick
        UpdateTrackbar()
    End Sub

    Private Sub Timer2_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Timer2.Tick
        If SlotAmp >= 0 Then
            txt_Amplitude.Text = Amplitude.ToString("0.000", Globalization.CultureInfo.InvariantCulture)
        End If
        If SlotShift >= 0 Then
            txt_Shift.Text = Shift.ToString("0.000", Globalization.CultureInfo.InvariantCulture)
        End If
        If SlotFreq >= 0 Then
            txt_Frequency.Text = Frequency.ToString("0.000", Globalization.CultureInfo.InvariantCulture)
        End If
        If SlotWaveform >= 0 Then
            Dim NewWaveformIndex As Int32 = CInt(Slots.ReadSlot_NoNan(SlotWaveform))
            Dim nWaveforms As Int32 = cmb_Shape.Items.Count
            If NewWaveformIndex >= 0 And NewWaveformIndex < nWaveforms Then
                If NewWaveformIndex <> cmb_Shape.SelectedIndex Then
                    cmb_Shape.SelectedIndex = NewWaveformIndex
                    SetWaveShape()
                End If
            End If
        End If
    End Sub

End Class